unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OleCtrls, WMPLib_TLB, StdCtrls, ExtCtrls, ComCtrls, Menus,
  FindFile;

type
    TTimerThread = class(TThread)
    protected
        procedure Kick();
        procedure Execute; override;
    end;

  TMainForm = class(TForm)
    OpenDialog1: TOpenDialog;
    StatusBar: TStatusBar;
    MainMenu1: TMainMenu;
    N1: TMenuItem;
    MenuPlayerProp: TMenuItem;
    N5: TMenuItem;
    Panel1: TPanel;
    BtnMinus30: TButton;
    BtnMinus10: TButton;
    BtnPlus10: TButton;
    BtnPlus30: TButton;
    Panel2: TPanel;
    ComboList: TComboBox;
    PnlMain: TPanel;
    FPlayer: TWindowsMediaPlayer;
    Label1: TLabel;
    Label2: TLabel;
    LblOpenState: TLabel;
    LblPlayState: TLabel;
    MenuMinus30: TMenuItem;
    MenuMinus10: TMenuItem;
    MenuPlus10: TMenuItem;
    MenuPlus30: TMenuItem;
    N6: TMenuItem;
    MenuPrevMedia: TMenuItem;
    MenuNextMedia: TMenuItem;
    N7: TMenuItem;
    MenuTrashFile: TMenuItem;
    MenuVersion: TMenuItem;
    TrackBalance: TTrackBar;
    BtnSndVol: TButton;
    N2: TMenuItem;
    MenuConf: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FPlayerPlayStateChange(Sender: TObject; NewState: Integer);
    procedure FPlayerOpenStateChange(Sender: TObject; NewState: Integer);
    procedure FPlayerStatusChange(Sender: TObject);
    procedure FPlayerMediaChange(Sender: TObject; const Item: IDispatch);
    procedure FormDestroy(Sender: TObject);
    procedure MenuPlayerPropClick(Sender: TObject);
    procedure FPlayerClick(Sender: TObject; nButton, nShiftState: Smallint;
      fX, fY: Integer);
    procedure FormResize(Sender: TObject);
    procedure ComboListSelect(Sender: TObject);
    procedure BtnMinus30Click(Sender: TObject);
    procedure MenuNextMediaClick(Sender: TObject);
    procedure MenuTrashFileClick(Sender: TObject);
    procedure MenuVersionClick(Sender: TObject);
    procedure TrackBalanceChange(Sender: TObject);
    procedure BtnSndVolClick(Sender: TObject);
    procedure MenuConfClick(Sender: TObject);
  private
    FTimerThread: TTimerThread;
    FCounter: Integer;
    procedure UpdateForm();
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DropFiles;
  private
    procedure ShowStatus(const S: string; const index: integer);
    procedure SetPrevMedia();
    procedure SetNextMedia();
  public
    FSList: TStringList;
  end;

var
  MainForm: TMainForm;

implementation

uses
    ShellApi, ActiveX, Info, Prop, Config;

const
    PLAYER_STOP = '~';
    PLAYER_READY = '';
    INDEX_STATUS = 0;
    INDEX_TIME = 1;
    INDEX_SIZE = 2;
    INDEX_CANCEL = 3;

{$R *.dfm}

//=============================================================================
//  TTimerThread implementation
//=============================================================================

//-----------------------------------------------------------------------------
procedure TTimerThread.Kick();
begin
    MainForm.UpdateForm();
end;

//-----------------------------------------------------------------------------
procedure TTimerThread.Execute();
begin
    while not Terminated do
    begin
        Synchronize(Kick);
        Sleep(10);
    end;
end;

//=============================================================================
//  TMainForm implementation
//=============================================================================

//-----------------------------------------------------------------------------
//  GNXv[̃hbOAhhbv󂯕t悤ɂ
//  http://www2.big.or.jp/~osamu/Delphi/tips.cgi?index=0218.txt
procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
    inherited CreateParams(Params);
    Params.ExStyle := Params.EXStyle or WS_EX_ACCEPTFILES;
end;

//-----------------------------------------------------------------------------
//  RXgN^
//  http://hpcgi1.nifty.com/MADIA/DelphiBBS/wwwlng.cgi?print+200711/07110026.txt
//  WindowsMediaPlayer1ς̍Đ\ɂ́H
procedure TMainForm.FormCreate(Sender: TObject);
var
    LCore: IWMPCore;
    LPlayer4Disp: WMPLib_TLB.IWMPPlayer4Disp;
begin
    //  ݒǂݍ
    GConfig := TConfig.Create();
    GConfig.LoadConfig(ChangeFileExt(ParamStr(0), '.ini'));

    FSList := TStringList.Create;

    //  vC[̃C^[tF[Xo(?)
    LCore := FPlayer.ControlInterface;
    if Assigned(LCore) then
    begin
        //  StretchToFit ĂC^[tF[Xւ̎QƂo
        LCore.QueryInterface(WMPLib_TLB.IWMPPlayer4Disp, LPlayer4Disp);
        if Assigned(LPlayer4Disp) then
        begin
            //  ŁAݒ肷
            LPlayer4Disp.StretchToFit := True;
        end;
    end;

    FCounter := 0;
    FTimerThread := TTimerThread.Create(False);
    GFindFileThread.Resume();
end;

//-----------------------------------------------------------------------------
//  tH[TCYɒǏ]
//  BBS ւ̏(2007/12 ern )
procedure TMainForm.FormResize(Sender: TObject);
const
    IID_IOleInPlaceObject: TGUID = '{00000113-0000-0000-C000-000000000046}';
var
    IOIPObj: IOleInPlaceObject;
    WMPRect: TRect;
begin
    SetRect(WMPRect, 0, 0, PnlMain.Width, PnlMain.Height);
    IDispatch(FPlayer.OleObject).QueryInterface(IID_IOleInPlaceObject, IOIPObj);
    if Assigned(IOIPObj) then
    begin
        IOIPObj.SetObjectRects(WMPRect, WMPRect);
    end;
end;

//-----------------------------------------------------------------------------
//  fXgN^
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    FTimerThread.Free();
    //  LZ
    GFindFileThread.Cancel();

    //  ݒۑ
    GConfig.SaveConfig(ChangeFileExt(ParamStr(0), '.ini'));
    GConfig.Free();

    FSList.Free;
end;

//-----------------------------------------------------------------------------
//  GNXv[hbvꂽ̏
//  http://www2.big.or.jp/~osamu/Delphi/tips.cgi?index=0218.txt
procedure TMainForm.WMDropFiles(var Msg: TWMDropFiles);
var
    LPathName: Array[0..MAX_PATH] of Char;
    i, LCount: integer;
begin
    //  hbvꂽt@C̐擾
    LCount := DragQueryFile(Msg.Drop, DWORD(-1), LPathName, SizeOf(LPathName));
    for i := 0 to LCount - 1 do
    begin
        //  ꂼ̃t@C擾
        DragQueryFile(Msg.Drop, i, LPathName, SizeOf(LPathName));
        //  Xbhɒǉ
        GFindFileThread.PutPath(LPathName);
    end;
    //  hbv
    DragFinish(Msg.Drop);
end;

//-----------------------------------------------------------------------------
//  Xe[^Xo[ɃbZ[W\
procedure TMainForm.ShowStatus(const S: string; const index: integer);
begin
    StatusBar.Panels[index].Text := S;
end;

//-----------------------------------------------------------------------------
//  Ƃ߂
procedure TMainForm.BtnMinus30Click(Sender: TObject);
var
    LSender: TComponent;
begin
    LSender := TComponent(Sender);
    if (FPlayer.currentMedia <> nil) then
    begin
        FPlayer.controls.currentPosition
            := FPlayer.controls.currentPosition + LSender.Tag;
    end;
end;

//-----------------------------------------------------------------------------
//  ^C}[
procedure TMainForm.UpdateForm();
var
    s, s1, s2: string;
    d1, d2: Double;
    LBuf: TStringList;
    i: Integer;
begin
    //  hbvt@CfBAXgɒǉ
    if (GFindFileThread.ResultCount > 0) then
    begin
        //  f
        ShowStatus('[ESC] to Break', INDEX_CANCEL);
        if (GetAsyncKeyState(VK_ESCAPE) <> 0) then
        begin
            GFindFileThread.Cancel();
        end;

        LBuf := TStringList.Create();
        LBuf.Sorted := True;
        GFindFileThread.GetResult(LBuf, 10);
        for i := 0 to LBuf.Count - 1 do
        begin
            //  `FbN
            if (not GConfig.CheckFile(LBuf[i])) then Continue;

            //  vCXgɒǉƖΉ̃t@C͂B
            //  FLVȂǂ̕WȊO(?)̃t@CĐłꍇłB
            //  Ȃ̂ŁAOŃXgǗAĐsourceURLw肷B
            ComboList.Items.Add(LBuf[i]);

            //  AĐĂȂ΍ĐĂ݂
            if (FPlayer.status = PLAYER_READY)
            or (FPlayer.status = '') then
            begin
                SetNextMedia();
            end;
        end;
    end
    else
    begin
        ShowStatus('', INDEX_CANCEL);
    end;

    //  AĐĂȂ΍ĐĂ݂
    //  ̃ubNȂ flv t@CȂǕWT|[gÕt@CXLbv
    //  ƁAŌ̃fBAxĐĂ܂
    if (ComboList.ItemIndex <> -1) then
    begin
        if (FPlayer.status = PLAYER_READY)
        or (FPlayer.status = '') then
        begin
            FPlayer.controls.play();
            //SetNextMedia();
        end;
    end;

    //  Đ̃fBA̎ԂɊւ\
    if (FPlayer.currentMedia <> nil) then
    begin
        //  ݂̃fBAڂ̕bŎĐ(b)
        s1 := FPlayer.currentMedia.durationString;
        //  fBAړ݈̌ʒu擾(b)
        s2 := FPlayer.controls.currentPositionString;
        if (s2 = '') then s2 := '00:00';
        //  ݂̃fBAڂ̕bŎĐ
        d1 := FPlayer.currentMedia.duration;
        //  fBAړ݈̌ʒu擾(ύX\)
        d2 := FPlayer.controls.currentPosition;
        //  A
        s := Format('%s(%fb) / %s(%fb)', [s2, d2, s1, d1]);

        ShowStatus(s, INDEX_TIME);

        //  ĐIŎ̃fBAɈړ
        //  ĐĂȂ΍ĐĂ݂
        if (FPlayer.status = PLAYER_STOP)
        //or (FPlayer.status = PLAYER_READY)
        or (FPlayer.status = '') then
        begin
            SetNextMedia();
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  fBAύXꂽ
procedure TMainForm.FPlayerMediaChange(Sender: TObject; const Item: IDispatch);
var
    LMedia: IWMPMedia;
    LName: String;
    LWidth: Integer;
    LHeight: Integer;
begin
    LMedia := Item as IWMPMedia;
    LName := LMedia.name;
    LWidth := LMedia.imageSourceWidth;
    LHeight := LMedia.imageSourceHeight;
    Self.Caption := Format('%s %dx%D %s', [Application.Title, LWidth, LHeight, LName]);
end;

//-----------------------------------------------------------------------------
//  ĐE~
procedure TMainForm.FPlayerClick(Sender: TObject; nButton,
  nShiftState: Smallint; fX, fY: Integer);
var
    LRect: TRect;
begin
    //  ^񒆕t߂NbNĐEꎞ~؂ւ
    LRect := FPlayer.ClientRect;
    InflateRect(LRect, -50, -50);
    OffsetRect(LRect, 0, -50);
    if not (PtInRect(LRect, Point(fX, fY))) then
    begin
        inherited;
        Exit;
    end;

    if (FPlayer.controls.isAvailable['Play']) then
    begin
        FPlayer.controls.play();
    end
    else if (FPlayer.controls.isAvailable['Pause']) then
    begin
        FPlayer.controls.pause();
    end;
end;

//-----------------------------------------------------------------------------
//  t@CI
procedure TMainForm.ComboListSelect(Sender: TObject);
var
    LSize: Integer;
begin
    if (ComboList.ItemIndex = -1) then exit;
    //  Ôߍ̃fBA~Ă
    FPlayer.controls.stop;
    //  fBA؂ւ
    try
        FPlayer.URL := ComboList.Text;
        if (FPlayer.URL <> '') then
        begin
            LSize := FileSizeByFileSize(FPlayer.URL);
            ShowStatus(Format('%.2f[MB]', [LSize / (1024 * 1024)]), INDEX_SIZE);
            FPlayer.controls.play;
        end;
    except
        ;
    end;
end;

//-----------------------------------------------------------------------------
//  ЂƂÕfBA
procedure TMainForm.SetPrevMedia();
begin
    if (ComboList.Items.Count > 0) and (ComboList.ItemIndex > 0) then
    begin
        ComboList.ItemIndex := ComboList.ItemIndex - 1;
        ComboList.OnSelect(ComboList);
    end
    //  OȂ΃NA(ǃNAȂ)
    //  status uvɂȂ
    else
    begin
        ComboList.ItemIndex := -1;
        FPlayer.URL := '';
        FPlayer.currentMedia := nil;
    end;
end;

//-----------------------------------------------------------------------------
//  ЂƂ̃fBA
procedure TMainForm.SetNextMedia();
begin
    if (ComboList.Items.Count > ComboList.ItemIndex + 1) then
    begin
        ComboList.ItemIndex := ComboList.ItemIndex + 1;
        ComboList.OnSelect(ComboList);
    end
    //  Ȃ΃NA(ǃNAȂ)
    //  status uvɂȂ
    else
    begin
        FPlayer.URL := '';
        FPlayer.currentMedia := nil;
    end;
end;

//-----------------------------------------------------------------------------
//  fBAύX
procedure TMainForm.MenuNextMediaClick(Sender: TObject);
begin
    if (Sender = MenuPrevMedia) then SetPrevMedia()
    else
    if (Sender = MenuNextMedia) then SetNextMedia()
    ;
end;

//-----------------------------------------------------------------------------
//  Đ̃t@CS~Ɏ̂Ă
procedure TMainForm.MenuTrashFileClick(Sender: TObject);
begin
    if (ComboList.ItemIndex = -1) then exit;
    SendToRecycleBin(ComboList.Text);
    SetNextMedia();
end;

//-----------------------------------------------------------------------------
//  Đ
procedure TMainForm.FPlayerPlayStateChange(Sender: TObject; NewState: Integer);
var
    LText: string;
begin
    if (Low(PLAY_STATE_STR) <= NewState) and (NewState <= High(PLAY_STATE_STR)) then
    begin
        LText := PLAY_STATE_STR[NewState]
    end
    else
    begin
        LText := 'Unknown';
    end;
    LblPlayState.Caption := LText;
end;

//-----------------------------------------------------------------------------
//  fBȀ
procedure TMainForm.FPlayerOpenStateChange(Sender: TObject; NewState: Integer);
var
    LText: string;
begin
    if (Low(OPEN_STATE_STR) <= NewState) and (NewState <= High(OPEN_STATE_STR)) then
    begin
        LText := OPEN_STATE_STR[NewState]
    end
    else
    begin
        LText := 'Unknown';
    end;
    LblOpenState.Caption := LText;
end;

//-----------------------------------------------------------------------------
//  Windows Media FPlayer ̏ԕύX
procedure TMainForm.FPlayerStatusChange(Sender: TObject);
begin
    ShowStatus(FPlayer.status, INDEX_STATUS);
end;

//-----------------------------------------------------------------------------
//  vC[̐ݒ𒲂ׂ
procedure TMainForm.MenuPlayerPropClick(Sender: TObject);
begin
    PropForm.ShowModal();
end;

//-----------------------------------------------------------------------------
//  o[W
procedure TMainForm.MenuVersionClick(Sender: TObject);
begin
    OpenInfoForm();
end;

//-----------------------------------------------------------------------------
//  oX
procedure TMainForm.TrackBalanceChange(Sender: TObject);
begin
    FPlayer.settings.balance := TrackBalance.Position;
end;

//-----------------------------------------------------------------------------
//  TEhRg[N
procedure TMainForm.BtnSndVolClick(Sender: TObject);
begin
    SimpleShellExecute('sndvol32.exe');
end;

//-----------------------------------------------------------------------------
//  ݒ
procedure TMainForm.MenuConfClick(Sender: TObject);
begin
    ConfigForm.ShowModal();
end;

end.

